unit testAppl;

interface

uses
  TestFramework, SysUtils, Controls, Forms, Appl, Classes, Windows,
  IniFiles;

type
  TTestUnitAppl = class(TTestCase)
  published
    procedure TestFindPrevInstance;
    procedure TestShowPrevInstance;
    procedure TestFindAndShowPrevInstance;
    procedure TestReadFormRes;
    procedure TestFindForm;
    procedure TestRtmPPI;
    procedure TestDsgnPPI;
    procedure TestDsgnVsRtmPPI;
    procedure TestScaleValue;
    procedure TestChangeConstraints;
    procedure TestFuncScale;
    procedure TestGetIniFileName;
    procedure TestGetSectionName;
    procedure TestIniShowCmdAndFlags;
    procedure TestIniWidth;
    procedure TestIniHeight;
    procedure TestIniLeft;
    procedure TestIniTop;
    procedure TestSaveLoadFormState;
  end;

const
  testPPI = 16;
  testOldPPI = testPPI;
  testNewPPI = testPPI - 3;
  strFailedTest = 'failed test';

const
  Delay = 1000;

implementation

uses testUnit;

procedure TTestUnitAppl.TestFindPrevInstance;
var
  Test1, Test2: boolean;
  Temp: THandle;
begin
  Temp := Mutex;
  Test1 := not FindPrevInstance('TestMiniProg');
  Test2 := FindPrevInstance('TestMiniProg');
  StopPrevInstance;
  Check(Test1 and Test2, strFailedTest);
  Mutex := Temp;
end;

procedure TTestUnitAppl.TestShowPrevInstance;
begin
  Check(ShowPrevInstance('DUnit'), strFailedTest);
end;

procedure TTestUnitAppl.TestFindAndShowPrevInstance;
begin
  Check(FindAndShowPrevInstance('DUnit'), strFailedTest);
end;

procedure TTestUnitAppl.TestReadFormRes;
var
  List: TStringList;
  Test: boolean;
begin
  List := TStringList.Create;
  try
    ReadFormRes('TGUITestRunner', List);
    Test := List.Values['Caption'] = 'DUnit: An Xtreme testing framework';
  finally
    List.Free;
    Check(Test, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestFindForm;
var
  Test1, Test2, Test3: boolean;
begin
  Test1 := FindForm('testForm') = nil;
  testForm := TtestForm.Create(Application);
  try
    Test2 := FindForm('testForm') <> nil;
  finally
    testForm.Free;
    Test3 := FindForm('testForm') = nil;
  end;
  Check(Test1 and Test2 and Test3, strFailedTest);
end;

procedure TTestUnitAppl.TestRtmPPI;
var
  Test: boolean;
begin
  testForm := TtestForm.Create(Application);
  try
    Test := RtmPPI('testForm') = testForm.Canvas.TextHeight(strDelphiMagicText);
  finally
    testForm.Free;
    Check(Test, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestDsgnPPI;
var
  OldPPI, PPI: integer;
begin
  testForm := TtestForm.Create(Application);
  try
    OldPPI := DsgnPPI('testForm');
  finally
    testForm.Free;
    Check(OldPPI = testPPI, Format('DsgnPPI=%d, not %d', [OldPPI, testPPI]));
  end;
end;

procedure TTestUnitAppl.TestDsgnVsRtmPPI;
var
  Test: boolean;
  Text: string;
  OldPPI, NewPPI: integer;
begin
  Test := False;
  Text := strFailedTest;
  testForm := TtestForm.Create(Application);
  try
    OldPPI := RtmPPI('testForm');
    NewPPI := DsgnPPI('testForm');
    if (OldPPI > 0) and (NewPPI > 0) then
    begin
      Test := OldPPI = NewPPI;
      if not Test then
        Text := Format('DsgnPPI=%d not equal RtmPPI=%d DPI', [OldPPI, NewPPI]);
    end;
  finally
    testForm.Free;
    Check(Test, Text);
  end;
end;

procedure TTestUnitAppl.TestScaleValue;
var
  Test: integer;
begin
  Test := ScaleValue(testOldPPI, testNewPPI, testOldPPI);
  Check( Test = testNewPPI, Format('return wrong %d',[Test]));
end;

procedure TTestUnitAppl.TestChangeConstraints;
var
  Test1, Test2: boolean;
  Size1, Size2: integer;
begin
  Test1 := False;
  Test2 := False;
  Size1 := testOldPPI * 20;
  Size2 := ScaleValue(Size1, testNewPPI, testOldPPI);
  testForm := TtestForm.Create(Application);
  try
    testForm.Constraints.MaxHeight := Size1;
    testForm.Constraints.MinHeight := Size1;
    testForm.Constraints.MaxWidth := 0;
    testForm.Constraints.MinWidth := Size1;
    ChangeConstraints(testForm as TControl, testNewPPI, testOldPPI);
    Test1 := (testForm.Constraints.MaxHeight = Size2) and
      (testForm.Constraints.MinHeight = Size2) and
      (testForm.Constraints.MaxWidth = 0) and
      (testForm.Constraints.MinWidth = Size2);
    ChangeConstraints(testForm as TControl, testOldPPI, testNewPPI);
    Test2 := (testForm.Constraints.MaxHeight = Size1) and
      (testForm.Constraints.MinHeight = Size1) and
      (testForm.Constraints.MaxWidth = 0) and
      (testForm.Constraints.MinWidth = Size1);
  finally
    testForm.Free;
    Check(Test1 and Test2, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestFuncScale;
var
  OldPPI, NewPPI: integer;
begin
  testForm := TtestForm.Create(Application);
  try
    OldPPI := RtmPPI('testForm');
    NewPPI := DsgnPPI('testForm');
    if (OldPPI > 0) and (NewPPI > 0) and (OldPPI <> NewPPI) then
      testForm.ShowModal;
  finally
    testForm.Free;
    Check(True, 'very strange');
  end;
end;

procedure TTestUnitAppl.TestGetIniFileName;
begin
  Check(ExtractFileName(GetIniFileName) = 'testMiniProg' + cfgFileIniExt, strFailedTest);
end;

procedure TTestUnitAppl.TestGetSectionName;
begin
  Check(GetSectionName(Screen.Forms[0]) = 'GUITestRunner', strFailedTest);
end;

procedure TTestUnitAppl.TestIniShowCmdAndFlags;
var
  Place: TWindowPlacement;
  Section: string;
  Ini: TIniFile;
  Test1, Test2: boolean;
  Temp1, Temp2: integer;
begin
  Test1 := False;
  Test2 := False;
  testForm := TtestForm.Create(Application);
  try
    Section := GetSectionName(testForm);
    Ini := TIniFile.Create(GetIniFileName);
    Place.Length := SizeOf(Place);
    GetWindowPlacement(testForm.Handle, @Place);
    Temp1 := Place.showCmd;
    Temp2 := Place.flags;
    WriteIniShowCmd(Ini, Section, Place);
    WriteIniFlags(Ini, Section, Place);
    Place.showCmd := Temp1 + 1;
    Place.flags := Temp2 + 1;
    ReadIniShowCmd(Ini, Section, Place);
    ReadIniFlags(Ini, Section, Place);
    Test1 := Place.showCmd = Temp1;
    Test2 := Place.flags = Temp2;
  finally
    Ini.Free;
    testForm.Free;
    Check(Test1 and Test2, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestIniWidth;
var
  Place: TWindowPlacement;
  Section: string;
  Ini: TIniFile;
  Test1, Test2: boolean;
  Temp1, Temp2: integer;
  List: TStringList;
begin
  Test1 := False;
  Test2 := False;
  testForm := TtestForm.Create(Application);
  try
    Section := GetSectionName(testForm);
    Ini := TIniFile.Create(GetIniFileName);
    List := TStringList.Create;
    ReadFormRes(testForm.ClassName, List);
    Place.Length := SizeOf(Place);
    GetWindowPlacement(testForm.Handle, @Place);
    Temp1 := Place.rcNormalPosition.Right - Place.rcNormalPosition.Left;
    DeleteFile('testMiniProg.ini');
    ReadIniWidth(testForm, Ini, Section, Place, List, testNewPPI, testOldPPI);
    Test1 := testForm.Width = Temp1;
    Temp2 := Place.rcNormalPosition.Right;
    WriteIniWidth(testForm, Ini, Section, Place, testNewPPI, testOldPPI);
    ReadIniWidth(testForm, Ini, Section, Place, List, testNewPPI, testOldPPI);
    Test2 := Place.rcNormalPosition.Right = Temp2;
  finally
    Ini.Free;
    List.Free;
    testForm.Free;
    Check(Test1 and Test2, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestIniHeight;
var
  Place: TWindowPlacement;
  Section: string;
  Ini: TIniFile;
  Test1, Test2: boolean;
  Temp1, Temp2: integer;
  List: TStringList;
begin
  Test1 := False;
  Test2 := False;
  testForm := TtestForm.Create(Application);
  try
    Section := GetSectionName(testForm);
    Ini := TIniFile.Create(GetIniFileName);
    List := TStringList.Create;
    ReadFormRes(testForm.ClassName, List);
    Place.Length := SizeOf(Place);
    GetWindowPlacement(testForm.Handle, @Place);
    Temp1 := Place.rcNormalPosition.Bottom - Place.rcNormalPosition.Top;
    DeleteFile('testMiniProg.ini');
    ReadIniHeight(testForm, Ini, Section, Place, List, testNewPPI, testOldPPI);
    Test1 := testForm.Height = Temp1;
    Temp2 := Place.rcNormalPosition.Bottom;
    WriteIniHeight(testForm, Ini, Section, Place, testNewPPI, testOldPPI);
    ReadIniHeight(testForm, Ini, Section, Place, List, testNewPPI, testOldPPI);
    Test2 := Place.rcNormalPosition.Bottom = Temp2;
  finally
    Ini.Free;
    List.Free;
    testForm.Free;
    Check(Test1 and Test2, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestIniLeft;
var
  Place: TWindowPlacement;
  Section: string;
  Ini: TIniFile;
  Test1, Test2: boolean;
  Temp1, Temp2: integer;
  List: TStringList;
begin
  Test1 := False;
  Test2 := False;
  testForm := TtestForm.Create(Application);
  try
    Section := GetSectionName(testForm);
    Ini := TIniFile.Create(GetIniFileName);
    List := TStringList.Create;
    ReadFormRes(testForm.ClassName, List);
    Place.Length := SizeOf(Place);
    GetWindowPlacement(testForm.Handle, @Place);
    Temp1 := Place.rcNormalPosition.Left;
    DeleteFile('testMiniProg.ini');
    ReadIniLeft(Ini, Section, Place);
    Test1 := testForm.Left = Temp1;
    Temp2 := Place.rcNormalPosition.Left;
    WriteIniLeft(Ini, Section, Place);
    ReadIniLeft(Ini, Section, Place);
    Test2 := Place.rcNormalPosition.Left = Temp2;
  finally
    Ini.Free;
    List.Free;
    testForm.Free;
    Check(Test1 and Test2, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestIniTop;
var
  Place: TWindowPlacement;
  Section: string;
  Ini: TIniFile;
  Test1, Test2: boolean;
  Temp1, Temp2: integer;
  List: TStringList;
begin
  Test1 := False;
  Test2 := False;
  testForm := TtestForm.Create(Application);
  try
    Section := GetSectionName(testForm);
    Ini := TIniFile.Create(GetIniFileName);
    List := TStringList.Create;
    ReadFormRes(testForm.ClassName, List);
    Place.Length := SizeOf(Place);
    GetWindowPlacement(testForm.Handle, @Place);
    Temp1 := Place.rcNormalPosition.Top;
    DeleteFile('testMiniProg.ini');
    ReadIniTop(Ini, Section, Place);
    Test1 := testForm.Top = Temp1;
    Temp2 := Place.rcNormalPosition.Top;
    WriteIniTop(Ini, Section, Place);
    ReadIniTop(Ini, Section, Place);
    Test2 := Place.rcNormalPosition.Top = Temp2;
  finally
    Ini.Free;
    List.Free;
    testForm.Free;
    Check(Test1 and Test2, strFailedTest);
  end;
end;

procedure TTestUnitAppl.TestSaveLoadFormState;
const
  Delay = 1000;
var
  State1, State2, State3: TWindowPlacement;
  Test1, Test2, Test3, Test4, Test5, Test6: boolean;
  Start: integer;
  procedure Stop(ms: integer);
  begin
    Start := GetTickCount;
    while GetTickCount < (Start + ms) do
      Application.ProcessMessages;
  end;
begin
  testForm := TtestForm.Create(Application);
  LoadFormState(testForm);
  testForm.Show;
  DeleteFile('testMiniProg.ini');
  try
    Stop(Delay);
    State1.Length := SizeOf(State1);
    GetWindowPlacement(testForm.Handle, @State1);
    SaveFormState(testForm);
    State3.Length := SizeOf(State3);
    GetWindowPlacement(testForm.Handle, @State3);
    State3.rcNormalPosition.Left := State3.rcNormalPosition.Left - 100;
    State3.rcNormalPosition.Top := State3.rcNormalPosition.Top - 100;
    State3.rcNormalPosition.Right := State3.rcNormalPosition.Right - 100;
    State3.rcNormalPosition.Bottom := State3.rcNormalPosition.Bottom - 100;
    SetWindowPlacement(testForm.Handle, @State3);
    Stop(Delay);
    State3.flags := 2;
    State3.showCmd := 2;
    SetWindowPlacement(testForm.Handle, @State3);
    Stop(Delay);
    LoadFormState(testForm);
    State2.Length := SizeOf(State2);
    GetWindowPlacement(testForm.Handle, @State2);
    Test1 := State1.flags = State2.flags;
    Test2 := State1.showCmd = State2.showCmd;
    Test3 := State1.rcNormalPosition.Left = State2.rcNormalPosition.Left;
    Test4 := State1.rcNormalPosition.Top = State2.rcNormalPosition.Top;
    Test5 := State1.rcNormalPosition.Right = State2.rcNormalPosition.Right;
    Test6 := State1.rcNormalPosition.Bottom = State2.rcNormalPosition.Bottom;
    Stop(Delay);
  finally
    testForm.Free;
    Check(Test1 and Test2 and Test3 and Test4 and Test5 and Test6,
      'wrong save/load form state');
  end;
end;

initialization
  TestFramework.RegisterTest(TTestUnitAppl.Suite);
end.

